package cz.uhk.mikesji3.centrality.gui;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.List;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

import cz.uhk.mikesji3.centrality.measures.BetweennessC;
import cz.uhk.mikesji3.centrality.measures.ClosenessC;
import cz.uhk.mikesji3.centrality.measures.DegreeC;
import cz.uhk.mikesji3.centrality.measures.EigenvectorC;
import cz.uhk.mikesji3.centrality.measures.IMeasure;
import cz.uhk.mikesji3.centrality.measures.KatzC;
import cz.uhk.mikesji3.centrality.model.Edge;
import cz.uhk.mikesji3.centrality.model.Vertex;

public class CentralityBox extends JPanel {

	private static final long serialVersionUID = 1L;
	Gui gui;

	JButton btDegree;
	JButton btCloseness;
	JButton btBetweenness;
	JButton btEigenvector;
	JButton btKatz;
	JButton btExport;
	ActionListener alDegree;
	ActionListener alCloseness;
	ActionListener alBetweenness;
	ActionListener alEigenvector;
	ActionListener alKatz;

	JPanel pnButtons;
	JPanel pnExport;
	JTable resultTable;
	JTableHeader tableHeader;
	DefaultTableModel tableModel;

	public CentralityBox(Gui gui) {
		this.gui = gui;

		setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));
		addButtons();
		addTable();
		addExport();
	}

	private void addButtons() {
		pnButtons = new JPanel();
		pnButtons.setLayout(new GridLayout(5, 1));

		btDegree = new JButton("Degree");
		pnButtons.add(btDegree);
		alDegree = new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (gui.app.getG() != null) {
					IMeasure degreeC = new DegreeC(gui.app.getG());
					degreeC.calculate();
					degreeC.calculateRank(0);
					fillResultTable(degreeC.getVertices(), 0);
					changeColumnName("Degree");
					gui.changeSize(degreeC, 0);
				}
			}
		};
		btDegree.addActionListener(alDegree);

		btCloseness = new JButton("Closeness");
		pnButtons.add(btCloseness);
		alCloseness = new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (gui.app.getG() != null) {

					IMeasure closenessC = new ClosenessC(gui.app.getG());
					closenessC.calculate();
					closenessC.calculateRank(1);
					fillResultTable(closenessC.getVertices(), 1);
					changeColumnName("Closeness");
					gui.changeSize(closenessC, 1);
				}
			}
		};
		btCloseness.addActionListener(alCloseness);

		btBetweenness = new JButton("Betweenness");
		pnButtons.add(btBetweenness);
		alBetweenness = new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (gui.app.getG() != null) {
					IMeasure betweennessC = new BetweennessC(gui.app.getG());
					betweennessC.calculate();
					betweennessC.calculateRank(2);
					fillResultTable(betweennessC.getVertices(), 2);
					changeColumnName("Betweenness");
					gui.changeSize(betweennessC, 2);
				}
			}
		};
		btBetweenness.addActionListener(alBetweenness);

		btEigenvector = new JButton("Eigenvector");
		pnButtons.add(btEigenvector);
		alEigenvector = new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (gui.app.getG() != null) {
					IMeasure eigenvectorC = new EigenvectorC(gui.app.getG());
					eigenvectorC.calculate();
					eigenvectorC.calculateRank(3);
					fillResultTable(eigenvectorC.getVertices(), 3);
					changeColumnName("Eigenvector");
					gui.changeSize(eigenvectorC, 3);
				}
			}
		};
		btEigenvector.addActionListener(alEigenvector);

		btKatz = new JButton("Katz");
		pnButtons.add(btKatz);
		alKatz = new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (gui.app.getG() != null) {
					new KatzParameterFrame(gui, CentralityBox.this,
							gui.getLocation());
					gui.setEnabled(false);
				}
			}
		};
		btKatz.addActionListener(alKatz);

		add(pnButtons);
	}

	private void changeColumnName(String columnName) {
		TableColumnModel tcm = tableHeader.getColumnModel();
		TableColumn tc = tcm.getColumn(1);
		tc.setHeaderValue(columnName);
		tableHeader.repaint();
	}

	private void addTable() {
		tableModel = new DefaultTableModel() {
			private static final long serialVersionUID = 1L;

			@Override
			public Class<?> getColumnClass(int column) {
				if (column == 0 || column == 1) {
					return Integer.class;
				}
				return super.getColumnClass(column);
			}
		};

		resultTable = new JTable(tableModel);
		resultTable.setAutoCreateRowSorter(true);

		tableModel.addColumn("Vrchol");
		tableModel.addColumn("Centr.");
		tableModel.addColumn("Rank");
		resultTable.getColumnModel().getColumn(0).setPreferredWidth(50);
		resultTable.getColumnModel().getColumn(1).setPreferredWidth(50);
		resultTable.getColumnModel().getColumn(2).setPreferredWidth(50);

		DefaultTableCellRenderer centerRenderer = new DefaultTableCellRenderer();
		centerRenderer.setHorizontalAlignment(JLabel.CENTER);
		resultTable.getColumnModel().getColumn(0)
				.setCellRenderer(centerRenderer);
		resultTable.getColumnModel().getColumn(1)
				.setCellRenderer(centerRenderer);
		resultTable.getColumnModel().getColumn(2)
				.setCellRenderer(centerRenderer);

		tableHeader = resultTable.getTableHeader();
		JPanel tablePanel = new JPanel();
		tablePanel.setLayout(new BorderLayout());
		tablePanel.add(tableHeader, BorderLayout.NORTH);
		tablePanel.add(resultTable, BorderLayout.CENTER);

		add(tablePanel);
	}

	private void fillResultTable(List<Vertex> vertices, int type) {
		if (tableModel.getRowCount() > 0) {
			for (int i = tableModel.getRowCount() - 1; i > -1; i--) {
				tableModel.removeRow(i);
			}
		}
		for (int i = 0; i < vertices.size(); i++) {
			tableModel.insertRow(vertices.get(i).getId() - 1, new Object[] {
					vertices.get(i).getId(),
					vertices.get(i).getCentrality(type),
					vertices.get(i).getRank(type) });

		}
	}

	public void calculateKatz(float k) {
		IMeasure katzC = new KatzC(gui.app.getG(), k);
		katzC.calculate();
		katzC.calculateRank(4);
		fillResultTable(katzC.getVertices(), 4);
		changeColumnName("Katz");
		gui.changeSize(katzC, 4);
	}

	private void addExport() {
		pnExport = new JPanel();
		pnExport.setLayout(new GridLayout(1, 1));
		btExport = new JButton("Export");
		btExport.setSize(40, 2);
		btExport.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				try {
					export();
				} catch (IOException e1) {
					e1.printStackTrace();
				}
			}
		});
		pnExport.add(btExport);
		add(pnExport);
	}

	private void export() throws IOException {
		if (gui.app.getG() != null) {
			JFileChooser fileChooser = new JFileChooser();
			fileChooser.setDialogTitle("Zvol umístění a název souboru.");
			fileChooser.setCurrentDirectory(new File(System
					.getProperty("user.home")));
			fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
			int result = fileChooser.showSaveDialog(this);

			if(result == JFileChooser.APPROVE_OPTION){
				File file = fileChooser.getSelectedFile();
				exportGraphToTxt(file);
				exporthCentralityToCSV(file);
			}
		}
	}

	private void exportGraphToTxt(File file) throws FileNotFoundException {
		File textFile = new File(file.getPath() + ".txt");
		PrintWriter pw = new PrintWriter(textFile);
		pw.println(gui.app.getVerticesCount());
		List<Edge> edgeList = gui.app.getEdgeList();
		for (int i = 0; i < edgeList.size(); i++) {
			pw.println(edgeList.get(i).getVertex1() + "-"
					+ edgeList.get(i).getVertex2());
		}
		pw.close();
	}

	private void exporthCentralityToCSV(File file) throws IOException {
		File csvFile = new File(file.getPath() + ".csv");

		String newColumn = ";";
		String newLine = "\n";
		String header = "Vrchol;Degree;Closeness;Betweenness;Eigenvector;Katz 0.5";

		FileWriter fw = new FileWriter(csvFile);

		fw.append(header);
		fw.append(newLine);

		IMeasure degreeC = new DegreeC(gui.app.getG());
		degreeC.calculate();
		IMeasure closenessC = new ClosenessC(gui.app.getG());
		closenessC.calculate();
		IMeasure betweenessC = new BetweennessC(gui.app.getG());
		betweenessC.calculate();
		IMeasure eigenvectorC = new EigenvectorC(gui.app.getG());
		eigenvectorC.calculate();
		IMeasure katzC = new KatzC(gui.app.getG(), (float) 0.5);
		katzC.calculate();

		for (int i = 0; i < degreeC.getVertices().size(); i++) {
			fw.append(Integer.toString(i + 1));
			fw.append(newColumn);
			fw.append(Float.toString(degreeC.getVertices().get(i)
					.getCentrality(0)));
			fw.append(newColumn);
			fw.append(Float.toString(closenessC.getVertices().get(i)
					.getCentrality(1)));
			fw.append(newColumn);
			fw.append(Float.toString(betweenessC.getVertices().get(i)
					.getCentrality(2)));
			fw.append(newColumn);
			fw.append(Float.toString(eigenvectorC.getVertices().get(i)
					.getCentrality(3)));
			fw.append(newColumn);
			fw.append(Float.toString(katzC.getVertices().get(i)
					.getCentrality(4)));
			fw.append(newLine);
		}

		fw.flush();
		fw.close();

	}
}
