package cz.uhk.mikesji3.centrality.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Shape;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.geom.AffineTransform;
import java.awt.geom.Ellipse2D;
import java.util.List;
import java.util.Set;

import javax.swing.JFrame;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

import org.apache.commons.collections15.Transformer;

import cz.uhk.mikesji3.centrality.App;
import cz.uhk.mikesji3.centrality.measures.IMeasure;
import cz.uhk.mikesji3.centrality.model.Vertex;
import edu.uci.ics.jung.algorithms.layout.KKLayout;
import edu.uci.ics.jung.algorithms.layout.Layout;
import edu.uci.ics.jung.visualization.VisualizationViewer;
import edu.uci.ics.jung.visualization.control.DefaultModalGraphMouse;
import edu.uci.ics.jung.visualization.control.GraphMouseListener;
import edu.uci.ics.jung.visualization.control.ModalGraphMouse.Mode;
import edu.uci.ics.jung.visualization.decorators.EdgeShape;
import edu.uci.ics.jung.visualization.decorators.ToStringLabeller;
import edu.uci.ics.jung.visualization.picking.PickedState;

public class Gui extends JFrame {

	private static final long serialVersionUID = 1L;
	App app;
	VisualizationViewer<Integer,Integer> vv;

	JPanel menuPanel;
	JPanel centralityBox;
	JPanel leftPanel;
	Integer selectedVertex;
	
	JMenuBar menuBar;
	JMenuItem miControls;
	
	double[] vertexSizeCoefficients;

	public Gui(App sgv) {
		app = sgv;
		selectedVertex = -1;
		
		setTitle("Míry centrality");
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setResizable(false);
        
        setMenu();
        
        menuPanel = new MenuPanel(this);
        leftPanel = new JPanel();
        leftPanel.setPreferredSize(new Dimension(520,400));
        leftPanel.add(menuPanel, BorderLayout.NORTH);
        getContentPane().add(leftPanel,BorderLayout.CENTER);
        
        centralityBox = new CentralityBox(this);
        getContentPane().add(centralityBox, BorderLayout.EAST);
        
        setPreferredSize(new Dimension(580,550));
        pack();
        setVisible(true);  
	}

	private void setMenu() {
        menuBar = new JMenuBar();
        miControls = new JMenuItem("Nápověda");
        miControls.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				new ControlsFrame(Gui.this, getLocation());
				setEnabled(false);
			}
		});
        menuBar.add(miControls);
        setJMenuBar(menuBar);
	}

	public void drawGraph(){
		Layout<Integer, Integer> layout = new KKLayout<Integer, Integer>(app.getG());
        layout.setSize(new Dimension(360,360));
        vv = new VisualizationViewer<Integer,Integer>(layout);
        
        vv.setPreferredSize(new Dimension(400,380)); 
        vv.getRenderContext().setVertexLabelTransformer(new ToStringLabeller<Integer>()); //zobrazí popisky vrcholů
        vv.getRenderContext().setEdgeShapeTransformer(new EdgeShape.Line<Integer,Integer>());
        
        vv.addGraphMouseListener(new GraphMouseListener<Integer>() {
			public void graphReleased(Integer arg0, MouseEvent arg1) {
				if(SwingUtilities.isRightMouseButton(arg1)&&arg0<100&&arg0!=selectedVertex){
					if(!app.getG().isNeighbor(arg0, selectedVertex)){
						app.getG().addEdge(app.getEdgesCount()+101, arg0, selectedVertex);
						app.setEdgesCount(app.getEdgesCount()+1);
						selectedVertex = -1;
					}
				}
			}
			public void graphPressed(Integer arg0, MouseEvent arg1) {
				if(SwingUtilities.isRightMouseButton(arg1)){
					selectedVertex=arg0;
				}
			}
			public void graphClicked(Integer arg0, MouseEvent arg1) {
			}
		});
        
        final PickedState<Integer> pickedState = vv.getPickedEdgeState();
        
        vv.addKeyListener(new KeyListener() {
			public void keyTyped(KeyEvent arg0) {
				
			}
			public void keyReleased(KeyEvent arg0) {
				
			}
			public void keyPressed(KeyEvent e) {
				if(e.getKeyCode()==KeyEvent.VK_DELETE){
					Set<Integer> pickedEdges=pickedState.getPicked();
					for(Integer edge:pickedEdges){
						app.getG().removeEdge(edge);
						vv.repaint();
					}
				}
			}
		});
		
		DefaultModalGraphMouse<Integer, Integer> modalMouse = new DefaultModalGraphMouse<Integer, Integer>();
		modalMouse.setMode(Mode.PICKING);
		vv.setGraphMouse(modalMouse);
        
        leftPanel.remove(leftPanel.getComponentAt(300, 300));
        leftPanel.add(vv, BorderLayout.CENTER);
        pack();
	}

	public void changeSize(IMeasure centrality, int centralityInt) {
		List<Vertex> vertices = centrality.getVertices();
		
		vertexSizeCoefficients = getSizeCoefficients(vertices, centralityInt);
		
		Transformer<Integer,Shape> vertexSize = new Transformer<Integer,Shape>(){
            public Shape transform(Integer i){
                Ellipse2D circle = new Ellipse2D.Double(-5, -5, 10, 10);
                return AffineTransform.getScaleInstance(vertexSizeCoefficients[i-1], vertexSizeCoefficients[i-1]).createTransformedShape(circle);
                }
        };
        
        vv.getRenderContext().setVertexShapeTransformer(vertexSize);
        vv.repaint();
	}

	private double[] getSizeCoefficients(List<Vertex> vertices, int centralityInt) {
		double[] vertexSizeCoefficients = new double[vertices.size()];
		double min = getMin(vertices, centralityInt);
		double max = getMax(vertices, centralityInt);
		double range = max - min;
		
		for(int i = 0; i < vertices.size(); i++){
			if(range!=0){
				double size = vertices.get(i).getCentrality(centralityInt) - min;
				double ratio = size / range;
				vertexSizeCoefficients[i] = 1 + ratio * 3;
			}
			else{
				vertexSizeCoefficients[i] = 1;
			}
		}
		
		return vertexSizeCoefficients;
	}

	private double getMax(List<Vertex> vertices, int centralityInt) {
		double max = vertices.get(0).getCentrality(centralityInt);
		for(int i = 0; i < vertices.size(); i++){
			if(vertices.get(i).getCentrality(centralityInt)>max){
				max = vertices.get(i).getCentrality(centralityInt);
			}
		}
		return max;
	}

	private double getMin(List<Vertex> vertices, int centralityInt) {
		double min = vertices.get(0).getCentrality(centralityInt);
		for(int i = 0; i < vertices.size(); i++){
			if(vertices.get(i).getCentrality(centralityInt)<min){
				min = vertices.get(i).getCentrality(centralityInt);
			}
		}
		return min;
	}
}
