package cz.uhk.mikesji3.centrality.gui;

import java.awt.Color;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.filechooser.FileNameExtensionFilter;

import cz.uhk.mikesji3.centrality.model.Edge;

public class MenuPanel extends JPanel {

	private static final long serialVersionUID = 1L;
	Gui gui;

	JLabel lbVertices;
	JLabel lbEdges;
	JSlider slVertices;
	JSlider slEdges;
	JLabel lbError;
	JButton btDraw;
	JButton btImport;

	public MenuPanel(Gui gui) {
		this.gui = gui;

		setLayout(new GridLayout(4, 2));

		lbVertices = new JLabel("Počet vrcholů: 5");
		add(lbVertices);
		lbEdges = new JLabel("Počet hran: 8");
		add(lbEdges);

		addSliders();
		addConfirmation();
	}

	private void addSliders() {
		slVertices = new JSlider(JSlider.HORIZONTAL, 2, 20, 5);
		ChangeListener lstVertices = new ChangeListener() {
			public void stateChanged(ChangeEvent arg0) {
				int n = slVertices.getValue();
				lbVertices.setText("Počet vrcholů: " + n);
				slEdges.setMaximum(n*(n-1)/2);
				gui.app.setVerticesCount(slVertices.getValue());
			}
		};
		slVertices.addChangeListener(lstVertices);
		add(slVertices);

		slEdges = new JSlider(JSlider.HORIZONTAL, 1, 10, 8);
		ChangeListener lstEdges = new ChangeListener() {
			public void stateChanged(ChangeEvent arg0) {
				lbEdges.setText("Počet hran: " + slEdges.getValue());
				gui.app.setEdgesCount(slEdges.getValue());
			}
		};
		slEdges.addChangeListener(lstEdges);
		add(slEdges);
	}

	private void addConfirmation() {
		btDraw = new JButton("Vykreslit graf");
		btImport = new JButton("Import");
		lbError = new JLabel();
		lbError.setVisible(false);
		lbError.setForeground(Color.RED);

		btDraw.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				gui.app.setEdgesCount(slEdges.getValue());
				int n = slVertices.getValue();
				int maxEdges = (n * (n - 1)) / 2;
				if (slEdges.getValue() <= maxEdges) {
					lbError.setVisible(false);
					gui.app.buildRandomGraph();
					gui.drawGraph();
				} else {
					lbError.setText("Max počet hran hran: " + maxEdges);
					lbError.setVisible(true);
				}
			}
		});

		btImport.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				importGraph();
			}
		});

		add(btDraw);
		add(btImport);
		add(lbError);
	}

	private void importGraph() {
		File edgeFile = getEdgeFile();
		if (edgeFile != null) {
			try {
				int verticesCount = getVerticesCount(edgeFile);
				List<Edge> edgeList = getEdgeList(edgeFile);
				gui.app.buildGraph(verticesCount, edgeList);
				gui.drawGraph();
				lbError.setVisible(false);
			} catch (Exception e) {
				lbError.setText("Nevhodný soubor, viz nápověda");
				lbError.setVisible(true);
			}
		}
	}

	private File getEdgeFile() {
		JFileChooser fileChooser = new JFileChooser();
		FileNameExtensionFilter filter = new FileNameExtensionFilter(
				"TEXT FILES", "txt", "text");
		fileChooser.setFileFilter(filter);
		fileChooser.setCurrentDirectory(new File(System
				.getProperty("user.home")));
		int result = fileChooser.showOpenDialog(this);
		File edgeFile = null;
		if (result == JFileChooser.APPROVE_OPTION) {
			edgeFile = fileChooser.getSelectedFile();
		}
		return edgeFile;
	}

	private int getVerticesCount(File edgeFile) throws IOException {
		FileReader fr = new FileReader(edgeFile);
		BufferedReader br = new BufferedReader(fr);
		String line = br.readLine();
		int verticesCount = Integer.parseInt(line);
		br.close();
		return verticesCount;
	}

	private List<Edge> getEdgeList(File edgeFile) throws NumberFormatException, IOException {
		List<Edge> edgeList = new ArrayList<Edge>();
		FileReader fr = new FileReader(edgeFile);
		BufferedReader br = new BufferedReader(fr);
		String line = br.readLine();

		while ((line = br.readLine()) != null) {
			String[] array = line.split("-");
			Edge edge = new Edge(Integer.parseInt(array[0]),
					Integer.parseInt(array[1]));
			edgeList.add(edge);
		}
		
		br.close();
		return edgeList;
	}
}
