package cz.uhk.mikesji3.centrality.measures;

import cern.colt.matrix.DoubleMatrix1D;
import cern.colt.matrix.DoubleMatrix2D;
import cern.colt.matrix.impl.DenseDoubleMatrix2D;
import cern.colt.matrix.linalg.EigenvalueDecomposition;
import edu.uci.ics.jung.graph.Graph;

public class EigenvectorC extends AbstractMeasure implements IMeasure {

	double[][] matrix;
	float maxEigenValue;
	float[] eigenvector;
	DoubleMatrix2D doubleMatrix;
	EigenvalueDecomposition ed;

	public EigenvectorC(Graph<Integer, Integer> g) {
		super(g);
		createAdjacencyMatrix();

		doubleMatrix = new DenseDoubleMatrix2D(matrix);
		ed = new EigenvalueDecomposition(doubleMatrix);
	}

	private void createAdjacencyMatrix() {
		int verticesNr = g.getVertexCount();
		matrix = new double[verticesNr][verticesNr];
		for (int i = 0; i < verticesNr; i++) {
			for (int j = 0; j < verticesNr; j++) {
				if (g.isNeighbor(i + 1, j + 1)) {
					matrix[i][j] = 1;
				} else {
					matrix[i][j] = 0;
				}
			}
		}
	}

	public void calculate() {
		int position = getMaxEigenValuePos();
		setEigenvector(position);
		
		for (int i = 0; i < vertices.size(); i++) {
			if(g.getNeighborCount(i+1)==0){
				vertices.get(i).setCentrality(0, 3);
			}
			else{
				float sum = 0;
				for(Integer n:g.getNeighbors(i+1)){
					sum = sum + eigenvector[n-1];
				}
				float centrality = 1/maxEigenValue * sum;
				float r = (float) Math.round(centrality * 100) / 100;
				if(r>0){
					vertices.get(i).setCentrality(r, 3); // 3 = Eigenvector
				}
				else{
					vertices.get(i).setCentrality(-r, 3); // 3 = Eigenvector
				}
			}
		}
	}

	private void setEigenvector(int position) {
		DoubleMatrix2D eigenvectorsAll = ed.getV();
		eigenvector = new float[vertices.size()];
		for(int i = 0; i < vertices.size(); i++){
			eigenvector[i]=(float) eigenvectorsAll.get(i, position);
		}
	}

	private int getMaxEigenValuePos() {
		DoubleMatrix1D eigenValuesMatrix = ed.getRealEigenvalues();
		maxEigenValue = 0;
		int position = -1;
		for(int i = 0; i < eigenValuesMatrix.size(); i++){
			if(eigenValuesMatrix.get(i)>maxEigenValue){
				maxEigenValue = (float) eigenValuesMatrix.get(i);
				position = i;
			}
		}
		return position;
	}
}
