package cz.uhk.mikesji3.centrality.measures;

import cern.colt.matrix.DoubleMatrix1D;
import cern.colt.matrix.DoubleMatrix2D;
import cern.colt.matrix.impl.DenseDoubleMatrix1D;
import cern.colt.matrix.impl.DenseDoubleMatrix2D;
import cern.colt.matrix.linalg.Algebra;

import cz.uhk.mikesji3.centrality.algorithms.ShortestPaths;

import edu.uci.ics.jung.graph.Graph;

public class KatzC extends AbstractMeasure implements IMeasure{
	
	float k;
	ShortestPaths sp;
	double[][] A;

	public KatzC(Graph<Integer, Integer> g, float k) {
		super(g);
		sp = new ShortestPaths(g);
		createAdjacencyMatrix();
		this.k = k;
	}
	
	public void calculate() {
		double[] ones = new double[vertices.size()];
		for (int i = 0; i < vertices.size(); i++) {
			ones[i] = 1;
		}
		int[][] I = new int[vertices.size()][vertices.size()];
		for (int i = 0; i < vertices.size(); i++) {
			for (int j = 0; j < vertices.size(); j++) {
				if (i != j) {
					I[i][j] = 0;
				} else {
					I[i][j] = 1;
				}
			}
		}

		for (int j = 0; j < vertices.size(); j++) {
			for (int l = 0; l < vertices.size(); l++) {
				A[j][l] = k * A[j][l];
			}
		}

		for (int j = 0; j < vertices.size(); j++) {
			for (int l = 0; l < vertices.size(); l++) {
				A[j][l] = I[j][l] - A[j][l];
			}
		}
		
		DoubleMatrix2D matrix = new DenseDoubleMatrix2D(A);
		Algebra alg = new Algebra();
		matrix = alg.inverse(matrix);

		for (int j = 0; j < vertices.size(); j++) {
			for (int l = 0; l < vertices.size(); l++) {
				matrix.set(j, l, matrix.get(j, l) - I[j][l]);
			}
		}

		DoubleMatrix1D onesMatrix = new DenseDoubleMatrix1D(ones);
		DoubleMatrix1D centralityMatrix = alg.mult(matrix, onesMatrix);
		
		if(centralityMatrix.get(0)<0){
			for(int i = 0; i<centralityMatrix.size();i++){
				centralityMatrix.set(i, centralityMatrix.get(i)*-1);
			}
		}

		for (int i = 0; i < centralityMatrix.size(); i++) {
			float c = (float) centralityMatrix.get(i);
			float r = (float) Math.round(c * 100) / 100;
			vertices.get(i).setCentrality(Math.abs(r), 4);
		}
	}

	private void createAdjacencyMatrix() {
		int verticesNr = g.getVertexCount();
		A = new double[verticesNr][verticesNr];
		for (int i = 0; i < verticesNr; i++) {
			for (int j = 0; j < verticesNr; j++) {
				if (g.isNeighbor(i + 1, j + 1)) {
					A[i][j] = 1;
				} else {
					A[i][j] = 0;
				}
			}
		}
	}
}
